################################################################################
#  Copyright (C) 2009 Johan Liebert, Mantycore, Hedger, Rusanon                #
#  < anoma.team@gmail.com ; http://orphereus.anoma.ch >                        #
#                                                                              #
#  This file is part of Orphereus, an imageboard engine.                       #
#                                                                              #
#  This program is free software; you can redistribute it and/or               #
#  modify it under the terms of the GNU General Public License                 #
#  as published by the Free Software Foundation; either version 2              #
#  of the License, or (at your option) any later version.                      #
#                                                                              #
#  This program is distributed in the hope that it will be useful,             #
#  but WITHOUT ANY WARRANTY; without even the implied warranty of              #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               #
#  GNU General Public License for more details.                                #
#                                                                              #
#  You should have received a copy of the GNU General Public License           #
#  along with this program; if not, write to the Free Software                 #
#  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. #
################################################################################

"""The base Controller API

Provides the BaseController class for subclassing, and other objects
utilized by Controllers.
"""
import pylons as __pylons
from pylons import url, c, cache, config, g, request, response, session
from pylons.controllers import WSGIController
from pylons.controllers.util import abort, etag_cache
from pylons.decorators import jsonify, validate
from pylons.i18n import _, ungettext, N_
#from pylons.templating import render
from pylons.templating import render_mako as render

from pylons.controllers.util import redirect

def redirect_to(*args, **kwargs):
    return redirect(__pylons.url(*args, **kwargs))

import Orphereus.lib.helpers as h
import Orphereus.model as model

from Orphereus.model import meta
import time
from hotshot import Profile

import logging
log = logging.getLogger(__name__)

class BaseController(WSGIController):
    def __call__(self, environ, start_response):
        """Invoke the Controller"""
        # WSGIController.__call__ dispatches to the Controller method
        # the request is routed to. This routing information is
        # available in environ['pylons.routes_dict']
        try:
            if g.OPT.requestProfiling:
                pf = Profile(g.OPT.profileDumpFile)
                response = pf.runcall(WSGIController.__call__, self, environ, start_response)
                pf.close()
                return response
            else:
                return WSGIController.__call__(self, environ, start_response)
        finally:
            meta.Session.remove()

# Include the '_' function in the public names
__all__ = [__name for __name in locals().keys() if not __name.startswith('_') \
           or __name == '_']
