################################################################################
#  Copyright (C) 2009 Johan Liebert, Mantycore, Hedger, Rusanon                #
#  < anoma.team@gmail.com ; http://orphereus.anoma.ch >                        #
#                                                                              #
#  This file is part of Orphereus, an imageboard engine.                       #
#                                                                              #
#  This program is free software; you can redistribute it and/or               #
#  modify it under the terms of the GNU General Public License                 #
#  as published by the Free Software Foundation; either version 2              #
#  of the License, or (at your option) any later version.                      #
#                                                                              #
#  This program is distributed in the hope that it will be useful,             #
#  but WITHOUT ANY WARRANTY; without even the implied warranty of              #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               #
#  GNU General Public License for more details.                                #
#                                                                              #
#  You should have received a copy of the GNU General Public License           #
#  along with this program; if not, write to the Free Software                 #
#  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. #
################################################################################

import sqlalchemy as sa
from sqlalchemy import orm

from Orphereus.model import meta
import datetime

import logging
log = logging.getLogger(__name__)

from Orphereus.model import meta

def t_settings_init(dialectProps):
    return sa.Table("settings", meta.metadata,
        sa.Column("id"       , sa.types.Integer, sa.Sequence('settings_id_seq'), primary_key = True),
        sa.Column("name"     , sa.types.String(64), nullable = False, unique = True),
        sa.Column("value"    , sa.types.UnicodeText, nullable = False)
        )

class Setting(object):
    def __init__(self, name, value):
        self.name = name
        self.value = value

    def setValue(self, value):
        self.value = value
        meta.Session.commit()

    def delete(self):
        meta.Session.delete(self)
        meta.Session.commit()

    @staticmethod
    def create(name, value):
        setting = Setting(name, value)
        meta.Session.add(setting)
        meta.Session.commit()
        return setting

    @staticmethod
    def clearAll():
        map(lambda obj: obj.delete(), Setting.query.all())

    @staticmethod
    def getAll():
        return Setting.query.all()

    @staticmethod
    def getSection(name):
        return Setting.query.filter(Setting.name.like('%s.%%' % name))

    @staticmethod
    def getSetting(name):
        return Setting.query.filter(Setting.name == name).first()

